package services

import (
	"leafdev.top/Leaf/leaf-library-3/internal/base/logger"
	"leafdev.top/Leaf/leaf-library-3/internal/services/auth"
	"leafdev.top/Leaf/leaf-library-3/internal/services/collection"
	"leafdev.top/Leaf/leaf-library-3/internal/services/document"
	"leafdev.top/Leaf/leaf-library-3/internal/services/jwks"
	"leafdev.top/Leaf/leaf-library-3/internal/services/stream"
	"leafdev.top/Leaf/leaf-library-3/internal/services/workspace"

	"github.com/google/wire"
)

type Service struct {
	logger     *logger.Logger
	Jwks       *jwks.JWKS
	Auth       *auth.Service
	Stream     *stream.Service
	Workspace  *workspace.Service
	Collection *collection.Service
	Document   *document.Service
}

var Provide = wire.NewSet(
	jwks.NewJWKS,
	auth.NewService,
	stream.NewService,
	workspace.NewService,
	collection.NewService,
	document.NewService,
	NewService,
)

func NewService(
	logger *logger.Logger,
	jwks *jwks.JWKS,
	auth *auth.Service,
	stream *stream.Service,
	workspace *workspace.Service,
	collection *collection.Service,
	document *document.Service,
) *Service {
	return &Service{
		logger,
		jwks,
		auth,
		stream,
		workspace,
		collection,
		document,
	}
}
